<?php

namespace Teknisa\MAN\Services;

use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Doctrine\ORM\EntityManager;
use Teknisa\MAN\Utilities;
use \Datetime;

class ImportacaoConsumidorService {
	
	protected $entityManager;

	public function __construct(\Doctrine\ORM\EntityManager $entityManager)  {
		$this->entityManager = $entityManager;
	}
	
	public function findColumnsAndPositions($CDLAYOUTIMP){
		$params = array(':CDLAYOUTIMP'  => $CDLAYOUTIMP);
		return $this->entityManager->getConnection()->fetchAll(self::SELECT_COLUMNS_AND_POS, $params);
	}
	
	public function findLayoutFeatures ($CDLAYOUTIMP){
		$params = array(':CDLAYOUTIMP'  => $CDLAYOUTIMP);
		return $this->entityManager->getConnection()->fetchAll(self::FIND_LAYOUT_FEATURES, $params);
	}
	
	public function getImportaNegativo ($CDFILIAL) {
		$params = array(':CDFILIAL'  => $CDFILIAL);
		return $this->entityManager->getConnection()->fetchAll(self::GET_IDIMPBLOLANCNEG, $params);
	}

	// Função para buscar dados de validação do layout
	public function getValLayout($CDLAYOUTIMP, $NRSEQTABELAY) {
		$sql = "
			SELECT CDLAYOUTIMP, NRSEQTABELAY, NRSEQVALLAY, NRPOSINIVALLAY, NRTAMVALLAY, DSVALVALLAY
			FROM VALLAYOUTIMP
			WHERE CDLAYOUTIMP = :P_CDLAYOUTIMP
			AND NRSEQTABELAY = :P_NRSEQTABELAY
		";
		$params = [
			':P_CDLAYOUTIMP' => $CDLAYOUTIMP,
			':P_NRSEQTABELAY' => $NRSEQTABELAY
		];
		return $this->entityManager->getConnection()->fetchAll($sql, $params);
	}


	// Função para validar o layout
	public function validaLayout($line, $valLayoutData) {
	    foreach ($valLayoutData as $valLayout) {
	        $valid = false;
	        $validValues = $valLayout['DSVALVALLAY'];
	        $length = intval($valLayout['NRTAMVALLAY']);
	        $startPos = intval($valLayout['NRPOSINIVALLAY']) - 1;
	
	        while (!empty($validValues)) {
	            // Extrai um pedaço de tamanho NRTAMVALLAY
	            $validValue = substr($validValues, 0, $length);
	
	            // Compara a linha na posição especificada com o pedaço extraído
	            if (substr($line, $startPos, $length) === $validValue) {
	                $valid = true;
	                break;
	            }
	
	            // Remove o pedaço extraído e o delimitador (delimitador é virgula)
	            $validValues = substr($validValues, $length + 1);
	        }
	
	        if (!$valid) {
	            return false;
	        }
	    }
	    return true;
	}


	private function initializeRecord() {
	    $fields = [
	        'ERRO', 'CDBAIRRO', 'CDCCUSCLIE', 'CDCLIENTE', 'CDCONSUMIDOR', 'CDEXCONSUMID', 'CDFAMILISALDIMP',
	        'CDIDCONSUMID', 'CDMUNICIPIO', 'CDPAIS', 'CDSENHACONS', 'CDSENHASITE', 'CDTIPOCONS', 'CDVENDEDOR',
	        'DSCOMPLENDECONS', 'DSEMAILCONS', 'DSENDECONS', 'DSREFERENDECONS', 'DTNASCCONS', 'IDBLOQSALDFS',
	        'IDCADCONFLIBCON', 'IDIMPCPFCUPOM', 'IDPERCONSPRODEX', 'IDTPSEALMOCO', 'IDTPSELMANHA', 'IDTPSELTARDE',
	        'IDTPVENDACONS', 'IDVERSALDCON', 'NMBAIRCONS', 'NMCCUSCLIE', 'NMCONSUMIDOR', 'NMLOGINCONS', 'NMRESPCONS',
	        'NRACESSOSITE', 'NRCELULARCONS', 'NRCEPCONS', 'NRCPFRESPCON', 'NRENDECONS', 'NRFAXCONS', 'NRRAMAL1CONS',
	        'NRRAMAL2CONS', 'NRRGCONSUMID', 'NRTELE2CONS', 'NRTELECONS', 'SGESTADO', 'VRAVICREDCONS', 'VRAVIDEBCONS',
	        'VRDIASUBCONS', 'VRLIMCREDCONS', 'VRLIMDEBCONS', 'VRMAXCONSMENSAL', 'VRMAXCREDCONS', 'VRMAXDEBCONS',
	        'VRMESUBCONS', 'VRPESUBCONS', 'VRSALDOCREDIMP', 'VRSUBSBASICO'
	    ];
	
	    $record = [];
	    foreach ($fields as $field) {
	        $record[$field] = '';
	    }
	    return $record;
	}

	public function processFile($decodedFile, $columnsAndPositions, $layoutFeatures, $codClienteInformado, $valLayoutData, $filename) {
	    $lines = explode("\n", $decodedFile);
	    $records = [];
	
	    // Extract layout features
	    $isDelimited = $layoutFeatures['IDSEPARCAMPO'] === 'D';
	    $delimiter = $layoutFeatures['IDCARACDELIM'];
	    $filial = $layoutFeatures['CDFILIAL'];
	    
	    $importaNegativo = $this->getImportaNegativo($filial);
		if (isset($importaNegativo[0]) && !empty($importaNegativo[0])) {
			$importaNegativo = $importaNegativo[0]['IDIMPBLOLANCNEG'] === 'S';
		}
	
	    foreach ($lines as $line) {
	        if (trim($line) == '') continue;
	
			if (!empty($valLayoutData) && !$this->validaLayout($line, $valLayoutData)) {
				continue;
			}
	
        	// Initialize record with all fields set to empty
	        $record = $this->initializeRecord();
		        
		    $inPosInicial = 0;
	        foreach ($columnsAndPositions as $col) {
	            $value = $this->processFieldValue($line, $col, $isDelimited, $delimiter, $columnsAndPositions, $inPosInicial);
	            $record[$col['NMCOLUNAIMP']] = $value;
	        }
	
			// Apply adjustments
	        $record = $this->applyAdjustments($record, $layoutFeatures);
		
	        // Validate CDCLIENTE
	        $validationResult = $this->validateCdCliente($record, $codClienteInformado, $filial);
	        $record['CDCLIENTE'] = $validationResult['CDCLIENTE'];
	        $record['IDVALIDO'] = $validationResult['IDVALIDO'];
	        $record['IDMOTIVO'] = $validationResult['IDMOTIVO'];
	
			// Validate CDTIPOCONS
        	$record['CDTIPOCONS'] = $this->validateCdTipoCons($record['CDTIPOCONS'], $filial);
	
	        // Add default values and validation
	        $this->applyDefaultValuesAndValidation($record);
			
			if ($importaNegativo && isset($record['VRLIMDEBCONS']) && $record['VRLIMDEBCONS'] !== '') {
			    // Tenta converter para número
			    $valor = floatval($record['VRLIMDEBCONS']);
			    
			    if ($valor > 0) {
			        $record['VRLIMDEBCONS'] = $valor * -1;
			    }
			}
	        
	        // coloca valores (que serao utilizados posteriormente) nos records, para serem acessados facilmente
	        $record['CDFILIAL'] = $filial;
	        $record['IDINACONSARQIMP'] = $layoutFeatures['IDINACONSARQIMP'];
	        $record['IDPARALISAIMPARQ'] = $layoutFeatures['IDPARALISAIMPARQ'];
	        $record['CDLAYOUTIMP'] = $layoutFeatures['CDLAYOUTIMP'];
	        $record['NRSEQTABELAY'] = $layoutFeatures['NRSEQTABELAY'];
	        $record['FILENAME'] = $filename;
 	        
	        // Filter out empty fields
	        $record['fullRecord'] = $record;
	        $filteredRecord = array_filter($record, function($value) {
	            return $value !== '';
	        });
	
	        $records[] = $filteredRecord;
	    }
	
	    return $records;
	}

	private function processFieldValue(&$line, $col, $isDelimited, $delimiter, &$compArq, &$inPosInicial) {
	    $startPos = $col['NRPOSICOLLAY'] - 1;
	    $length = $col['NRTAMCOLLAY'];
	    $value = '';
	
	    // Valor padrão
	    if (!empty($col['DSDEFACOLLAY'])) {
	        $value = $col['DSDEFACOLLAY'];
	    } else {
        // Extração do valor do campo no arquivo
        if ($isDelimited) {
        	// logica se tiver delimitador
            if ($inPosInicial <= strlen($line)) {
	                $delimiterPos = strpos($line, $delimiter, $inPosInicial);
	                if ($delimiterPos !== false) {
	                    $value = trim(substr($line, $inPosInicial, $delimiterPos - $inPosInicial));
	                    $inPosInicial = $delimiterPos + 1; // Move a posição inicial para após o delimitador
	                } else {
	                    $value = trim(substr($line, $inPosInicial));
	                    $inPosInicial = strlen($line); // Move a posição inicial para além do final da linha
	                }
	            } else {
	                $value = '';
	            }

	        }  else {
	            // Lógica para campos não delimitados, utiliza a posição inicial e o tamanho do campo
	            $value = trim(substr($line, $startPos, $length));
	        }
	    }
	
	    // Specific check for CDCLIENTE to remove characters
	    if ($col['NMCOLUNAIMP'] === 'CDCLIENTE' && !empty($value)) {
	        $value = $this->removeCaracteres($value);
	    }
	    
	    
	    // Handle numeric fields with decimal places after extracting value
	    if ($col['IDTDADCOLLAY'] === 'N' && $col['NRCASASDECIM'] > 0 && strpos($value, ',') === false && strpos($value, '.') === false) {
	        $decimalPosition = strlen($value) - $col['NRCASASDECIM'];
	        if ($decimalPosition > 0) {
	            $value = substr($value, 0, $decimalPosition) . '.' . substr($value, $decimalPosition);
	        }
	    }
	
	    // Handle fields that can be null
	    if (in_array($col['NMCOLUNAIMP'], ['VRAVIDEBCONS', 'VRLIMDEBCONS', 'VRMAXDEBCONS', 'VRSUBSBASICO', 'VRMAXCONSMENSAL', 'VRAVICREDCONS', 'VRLIMCREDCONS', 'VRMAXCREDCONS'])) {
	        if (!empty($value)) {
	            $value = number_format((float)$value, 2, '.', '');
	        } else {
	            $value = '';
	        }
	    } else {
	        // Handle date fields
	        if ($col['IDTDADCOLLAY'] === 'D') {
	            if (!empty($value)) {
	                if (strpos($value, '/') === false) {
	                    $value = substr($value, 0, 2) . '/' . substr($value, 2, 2) . '/' . substr($value, 4, 4);
	                }
	                $dateTime = \DateTime::createFromFormat('d/m/Y', $value);
	                
	                if ($dateTime) {
			            $value = $dateTime->format('d/m/Y');
			        } else {
			        	$value = '01/01/1900';
			        }
	            } else {
	                $value = '01/01/1900';
	            }
	        } elseif ($col['IDTDADCOLLAY'] === 'N') {
	            // Handle numeric fields
	            if (!empty($value)) {
	                $value = (float)$value;
	            } else {
	                $value = 0;
	            }
	        } else {
	            // Handle default concatenation
	            if (!empty($value)) {
	                $value = $col['DSCONCATESQ'] . $value . $col['DSCONCATDIR'];
	            }
	        }
	    }
	
	    return $value;
	}
    
    
    private function removeZeroEsq($str) {
	    while (strlen($str) > 0 && $str[0] === '0') {
	        $str = substr($str, 1);
	    }
	    return $str;
	}

    
    // ajusta o tamanho de cada campo conforme especificado pelo delphi, adicionando 0s a esquerda
    private function ajusta($stString, $inTamStr) {
	    $stString = trim($stString);
	    if (strlen($stString) != $inTamStr) {
	        if (!empty($stString)) {
	            for ($i = 0; $i < strlen($stString); $i++) {
	                if (!is_numeric($stString[$i]) && $stString[$i] !== '.') {
	                    return $stString;
	                }
	            }
	            while (strlen($stString) < $inTamStr) {
	                $stString = '0' . $stString;
	            }
	        }
	    }
	    return $stString;
	}
	
	private function applyAdjustments($record, $layoutFeatures) {
	    $record['CDPAIS'] = $this->ajusta($record['CDPAIS'], 4);
	    $record['CDMUNICIPIO'] = $this->ajusta($record['CDMUNICIPIO'], 4);
	    $record['CDCONSUMIDOR'] = strtoupper($this->ajusta($record['CDCONSUMIDOR'], 21));
	    $record['CDCCUSCLIE'] = $this->ajusta($record['CDCCUSCLIE'], 10);
	    
	    
	    // CDIDCONSUMID
	    if ($layoutFeatures['IDREMOVEZEROESQ'] === 'S') {
	        $record['CDIDCONSUMID'] = $this->removeZeroEsq(strtoupper($record['CDIDCONSUMID']));
	    } else {
	        $record['CDIDCONSUMID'] = strtoupper($record['CDIDCONSUMID']);
	    }
	    
	    return $record;
	}


    private function removeCaracteres($str) {
	    $caracteres = './-';
	    for ($i = 0; $i < strlen($str); $i++) {
	        if (strpos($caracteres, $str[$i]) !== false) {
	            $str = substr_replace($str, '', $i, 1);
	            $i--; // Decrementar o índice para compensar a remoção
	        }
	    }
	    return $str;
	}
	
	private function applyDefaultValuesAndValidation(&$record) {
        // Example validation similar to Delphi code
        if (empty($record['DTNASCCONS'])) {
            $record['DTNASCCONS'] = '01/01/1900';
        }

        $record['IDBLOQSALDFS'] = !empty($record['IDBLOQSALDFS']) ? strtoupper($record['IDBLOQSALDFS']) : 'N';
        $record['IDVERSALDCON'] = !empty($record['IDVERSALDCON']) ? strtoupper($record['IDVERSALDCON']) : 'N';
        $record['IDTPVENDACONS'] = !empty($record['IDTPVENDACONS']) ? strtoupper($record['IDTPVENDACONS']) : '7';

        if (empty($record['CDCONSUMIDOR']) && !empty($record['CDEXCONSUMID'])) {
            $record['CDCONSUMIDOR'] = $record['CDEXCONSUMID'];
        }

        $record['IDTPSELMANHA'] = $this->validateField($record, 'IDTPSELMANHA', 'N');
        $record['IDTPSELTARDE'] = $this->validateField($record, 'IDTPSELTARDE', 'N');
        $record['IDTPSEALMOCO'] = $this->validateField($record, 'IDTPSEALMOCO', 'N');
    }

    private function validateField($record, $field, $default) {
        if (!empty($record[$field])) {
            return strtoupper($record[$field]) === 'S' ? 'S' : $default;
        }
        return $default;
    }
    
    private function validateCdCliente($record, $codClienteInformado, $filial) {
	    $validationResult = [
	        'CDCLIENTE' => '',
	        'IDVALIDO' => 'S',
	        'IDMOTIVO' => ''
	    ];
	
	    if ($codClienteInformado) {
	        $validationResult['CDCLIENTE'] = $codClienteInformado;
	        return $validationResult;
	    }
	
	    $cdCliente = $record['CDCLIENTE'];
	
	    if (empty($cdCliente)) {
	        $validationResult['CDCLIENTE'] = 'VAZIO';
	        $validationResult['IDVALIDO'] = 'N';
	        return $validationResult;
	    }
	
	    if (strlen($cdCliente) > 20) {
	        return $this->processCdExClienFil($cdCliente, $filial);
	    }
	
	    return $this->validateCliente($cdCliente, $filial);
	}
	
	private function validateCliente($cdCliente, $filial) {
	    $validationResult = [
	        'CDCLIENTE' => '',
	        'IDVALIDO' => 'S',
	        'IDMOTIVO' => ''
	    ];
	
	    $qryCdExClienFilResult = $this->queryCdExClienFil($cdCliente, $filial);
	    if (!empty($qryCdExClienFilResult)) {
	        $validationResult['CDCLIENTE'] = $qryCdExClienFilResult;
	        return $validationResult;
	    }
	
	    $qryNrInsJurClieResult = $this->queryNrInsJurClie($cdCliente, $filial);
	    if (!empty($qryNrInsJurClieResult)) {
	        $validationResult['CDCLIENTE'] = $qryNrInsJurClieResult;
	        return $validationResult;
	    }
	
	    $cdExternoResult = $this->queryCdExterno($cdCliente, $filial);
	    if (!empty($cdExternoResult)) {
	        $validationResult['CDCLIENTE'] = $cdExternoResult;
	        return $validationResult;
	    }
	
	    $qryClienteSemFilialResult = $this->queryClienteSemFilial($cdCliente);
	    if (!empty($qryClienteSemFilialResult)) {
	        $validationResult['CDCLIENTE'] = $cdCliente;
	        $validationResult['IDVALIDO'] = 'N';
	    	$validationResult['IDMOTIVO'] = 'CLI_FIL';
	        return $validationResult;
	    }
	
	    $validationResult['CDCLIENTE'] = $cdCliente;
	    $validationResult['IDVALIDO'] = 'N';
	    $validationResult['IDMOTIVO'] = 'NAO_ENC';
	    return $validationResult;
	}
	
	private function processCdExClienFil($cdCliente, $filial) {
	    $validationResult = [
	        'CDCLIENTE' => '',
	        'IDVALIDO' => 'S',
	        'IDMOTIVO' => ''
	    ];
	
	    $qryCdExClienFilResult = $this->queryCdExClienFil($cdCliente, $filial);
	    if (!empty($qryCdExClienFilResult)) {
	        $validationResult['CDCLIENTE'] = $qryCdExClienFilResult;
	        return $validationResult;
	    }
	
	    $qryCdExClienteResult = $this->queryCdExCliente($cdCliente, $filial);
	    if (!empty($qryCdExClienteResult)) {
	        $validationResult['CDCLIENTE'] = $qryCdExClienteResult;
	        return $validationResult;
	    }
	
	    $cdExternoResult = $this->queryCdExterno($cdCliente, $filial);
	    if (!empty($cdExternoResult)) {
	        $validationResult['CDCLIENTE'] = $cdExternoResult;
	        return $validationResult;
	    }
	
	    $qryClienteSemFilialResult = $this->queryClienteSemFilial($cdCliente);
	    if (!empty($qryClienteSemFilialResult)) {
	        $validationResult['CDCLIENTE'] = $cdCliente;
	        $validationResult['IDVALIDO'] = 'N';
	    	$validationResult['IDMOTIVO'] = 'CLI_FIL';
	        return $validationResult;
	    }
	
	    $validationResult['CDCLIENTE'] = $cdCliente;
	    $validationResult['IDVALIDO'] = 'N';
	    $validationResult['IDMOTIVO'] = 'NAO_ENC';
	    return $validationResult;
	}
	
	
	private function validateCdTipoCons($cdTipoCons, $filial) {
	    $adjustedCdTipoCons = $this->ajusta($cdTipoCons, 2);
	
	    if (!empty($adjustedCdTipoCons)) {
	        $sql = "SELECT CDTIPOCONS
	                FROM TIPOCONSFILI
	                WHERE ((CDFILIAL = :P_CDFILIAL) OR (:P_TCDFILIAL = 'T'))
	                AND CDTIPOCONSEX = :P_CDTIPOCONSEX";
	        $params = [
	            ':P_CDFILIAL' => $filial,
	            ':P_TCDFILIAL' => ($filial === 'T') ? 'T' : '',
	            ':P_CDTIPOCONSEX' => $adjustedCdTipoCons
	        ];
	        $result = $this->entityManager->getConnection()->fetchOne($sql, $params);
	
	        if ($result) {
	            return $result;
	        } else {
	            return $adjustedCdTipoCons;
	        }
	    } else {
	        return $adjustedCdTipoCons;
	    }
	}
	
	
	private function queryCdExClienFil($cdCliente, $filial) {
	    $sql = "SELECT CL.CDCLIENTE
	            FROM CLIENTE CL, CLIENFILIAL CF
	            WHERE (CL.CDCLIENTE = CF.CDCLIENTE)
	            AND (CF.CDEXCLIENFIL = :P_CDEXCLIENTE)
	            AND (CF.CDFILIAL = :P_CDFILIAL)
	            ORDER BY CL.CDCLIENTE";
	    $params = [
	        ':P_CDEXCLIENTE' => $cdCliente,
	        ':P_CDFILIAL' => $filial
	    ];
	    return $this->entityManager->getConnection()->fetchOne($sql, $params);
	}
	
	private function queryNrInsJurClie($cdCliente, $filial) {
	    $sql = "SELECT CL.CDCLIENTE
	            FROM CLIENTE CL, CLIENFILIAL CF
	            WHERE ((CL.CDEXCLIENTE = :P_CDEXCLIENTE) OR (CL.NRINSJURCLIE = :P_CDEXCLIENTE))
	            AND (CL.CDCLIENTE = CF.CDCLIENTE)
	            AND (CF.CDFILIAL = :P_CDFILIAL)
	            ORDER BY CL.CDCLIENTE";
	    $params = [
	        ':P_CDEXCLIENTE' => $cdCliente,
	        ':P_CDFILIAL' => $filial
	    ];
	    return $this->entityManager->getConnection()->fetchOne($sql, $params);
	}
	
	private function queryClienteSemFilial($cdCliente) {
	    $sql = "SELECT CDCLIENTE, NRINSJURCLIE
	            FROM CLIENTE
	            WHERE (CDCLIENTE = :P_CDCLIENTE) OR (NRINSJURCLIE = :P_NRINSJURCLIE)";
	    $params = [
	        ':P_CDCLIENTE' => $cdCliente,
	        ':P_NRINSJURCLIE' => $cdCliente
	    ];
	    return $this->entityManager->getConnection()->fetchOne($sql, $params);
	}
	
	private function queryCdExCliente($cdCliente, $filial) {
	    $sql = "SELECT CL.CDCLIENTE
	            FROM CLIENTE CL, CLIENFILIAL CF
	            WHERE (CL.CDEXCLIENTE = :P_CDEXCLIENTE)
	            AND (CL.CDCLIENTE = CF.CDCLIENTE)
	            AND (CF.CDFILIAL = :P_CDFILIAL)
	            ORDER BY CL.CDCLIENTE";
	    $params = [
	        ':P_CDEXCLIENTE' => $cdCliente,
	        ':P_CDFILIAL' => $filial
	    ];
	    return $this->entityManager->getConnection()->fetchOne($sql, $params);
	}
	
	private function queryCdExterno($cdCliente, $filial) {
	    $sql = "SELECT CDCLIENTE
	            FROM CODEXTCLIENTE
	            WHERE CDEXCLIENTEIMP = :P_CDEXCLIENTEIMP
	            AND CDFILIAL = :P_CDFILIAL";
	    $params = [
	        ':P_CDEXCLIENTEIMP' => $cdCliente,
	        ':P_CDFILIAL' => $filial
	    ];
	    return $this->entityManager->getConnection()->fetchOne($sql, $params);
	}

	
	const SELECT_COLUMNS_AND_POS = "
		SELECT
		  CDLAYOUTIMP, NRSEQTABELAY, IDFUNARQLAY, NRSEQCOLULAY, NRPOSICOLLAY, NRTAMCOLLAY, NRCASASDECIM,
		  DSDEFACOLLAY, IDGERSEQCOLU, IDTDADCOLLAY, IDGERCODCOLU, CDFRMTCOLLAY, NMTABELAIMP, NMCOLUNAIMP,
		  DSCONCATESQ, DSCONCATDIR
		FROM
		  COLLAYOUTIMP
		WHERE
		  CDLAYOUTIMP = :CDLAYOUTIMP
		ORDER BY
		  NRPOSICOLLAY
	";
	
	const FIND_LAYOUT_FEATURES = "
		SELECT DISTINCT
		  L.CDLAYOUTIMP, L.NMLAYOUTIMP, L.IDSEPARCAMPO, L.IDCARACDELIM,
		  F.IDINACONSARQIMP, CDFILIAL, F.IDPARALISAIMPARQ, F.IDREMOVEZEROESQ, F.NRSEQTABELAY
		FROM
		  LAYOUTIMP L, FUNLAYOUTIMP F
		WHERE
		  (L.CDLAYOUTIMP = F.CDLAYOUTIMP) AND
		  (F.IDFUNARQLAY = '008') AND
		  L.CDLAYOUTIMP = :CDLAYOUTIMP
		ORDER BY
		  L.CDLAYOUTIMP
	";
	//valida se é para importar o VRLIMDEBCONS ("Bloquear lançamentos quando o saldo for inferior a") negativo
	//se SIM, deve-se importar o VRLIMDEBCONS negativo
	
	const GET_IDIMPBLOLANCNEG = "
		SELECT IDIMPBLOLANCNEG FROM PARAVEND
		WHERE CDFILIAL  = :CDFILIAL
	";
}